==> mongodbsession-1.1.0.jar: mongodbsession/mongodb_session.clj <==
(ns mongodbsession.mongodb_session
  (:require [monger.collection :as db]
	    [ring.middleware.session.store :as ringstore])
  (:import [java.util UUID Date]))

(deftype MongodbStore [collection-name auto-key-change?]
  ringstore/SessionStore
  (read-session [_ key]
      (if-let [entity (and key
        ;;work around for (name ::keyword) -> "keyword"
        (reduce (fn [r [k v]] (assoc r (name k) v)) {} (db/find-map-by-id collection-name key)))]
		  entity {}))
  (write-session [_ key data]
		 ;;work around for (name ::keyword) -> "keyword"
		 (let [data (zipmap (map #(if (and (keyword? %) (namespace %))
					    (-> % str (.substring 1)) %)
					 (keys data))
				    (vals data))
		       entity (and key (db/find-map-by-id collection-name key))
		       key-change? (or (not entity) auto-key-change?)
		       newkey (if key-change?
				(str (UUID/randomUUID)) key)]
		   (if entity
		     (do (if key-change?
			   (do (db/remove-by-id collection-name key)
			       (db/insert collection-name
					      (assoc data :_id newkey :_date (:_date entity))))
			   (db/update-by-id collection-name newkey
					  (assoc data :_date (:_date entity))))
			 newkey)
		     (do (db/insert collection-name (assoc data :_id newkey :_date (Date.)))
			 newkey))))
  (delete-session [_ key]
		  (db/remove-by-id collection-name key)
		  nil))

(defn mongodb-store
  ([] (mongodb-store {}))
  ([opt]
    (let [collection-name (opt :collection-name :ring_sessions)
	   auto-key-change? (opt :auto-key-change? false)]
       (MongodbStore. "ring_sessions" auto-key-change?))))

==> mongodbsession-1.1.2.jar: mongodbsession/mongodb_session.clj <==
(ns mongodbsession.mongodb_session
  (:require [monger.collection :as db]
	    [ring.middleware.session.store :as ringstore])
  (:import [java.util UUID Date]))

(defn clean-key[key]
  (clojure.string/replace key "." (str (char 179))))

(defn unclean-key[key]
  (clojure.string/replace key (str (char 179)) "."))

(defn clean-data [data]
  (zipmap
    (map #(if (and (keyword? %) (namespace %))
            (-> % str (.substring 1) clean-key) %)
         (keys data))
    (vals data)))

(defn unclean-data [data]
  (zipmap
    (map #(if (and (keyword? %) (namespace %))
            (-> % str (.substring 1) unclean-key) %)
         (keys data))
    (vals data)))

(deftype MongodbStore [collection-name auto-key-change?]
  ringstore/SessionStore
  (read-session [_ key]
      (if-let [entity (and key
        (db/find-map-by-id collection-name key))]
		  (unclean-data entity) {}))
  (write-session [_ key data]
		 ;;work around for (name ::keyword) -> "keyword"
		 (let [data (clean-data data)
		       entity (and key (db/find-map-by-id collection-name key))
		       key-change? (or (not entity) auto-key-change?)
		       newkey (if key-change?
				(str (UUID/randomUUID)) key)]
		   (if entity
		     (do (if key-change?
			   (do (db/remove-by-id collection-name key)
			       (db/insert collection-name
					      (assoc data :_id newkey :_date (:_date entity))))
			   (db/update-by-id collection-name newkey
					  (assoc data :_date (:_date entity))))
			 newkey)
		     (do (db/insert collection-name (assoc data :_id newkey :_date (Date.)))
			 newkey))))
  (delete-session [_ key]
		  (db/remove-by-id collection-name key)
		  nil))

(defn mongodb-store
  ([] (mongodb-store {}))
  ([opt]
    (let [collection-name (opt :collection-name :ring_sessions)
	   auto-key-change? (opt :auto-key-change? false)]
       (MongodbStore. "ring_sessions" auto-key-change?))))
